from uuid import UUID

from sqlalchemy.exc import IntegrityError

from ..schemas.user_schema import UserWriteDTO
from ..uow.uow_base import UnitOfWork


class UserService:
    def __init__(self, uow: UnitOfWork):
        self.uow = uow

    async def get_all_users(self):
        async with self.uow:
            try:
                res = await self.uow.users.find_all()
            except IntegrityError as e:
                await self.uow.rollback()
                res = e._message()
            else:
                await self.uow.commit()
            finally:
                return res

    async def add_one(self, data: UserWriteDTO):
        res = None
        async with self.uow:
            try:
                res = await self.uow.users.add_one(data)
            except IntegrityError as e:
                await self.uow.rollback()
                res = e._message()
            else:
                await self.uow.commit()
            finally:
                return res

    async def get_user(self, id: UUID):
        async with self.uow:
            try:
                res = await self.uow.users.find_one(filter={"id": id})
            except IntegrityError as e:
                await self.uow.rollback()
                res = e._message()
            else:
                await self.uow.commit()
            finally:
                return res

    async def patch_one(self, id: UUID, data: UserWriteDTO):
        res = None
        async with self.uow:
            try:
                res = await self.uow.users.patch_one(filter={"id": id}, data=data)
            except IntegrityError as e:
                await self.uow.rollback()
                res = e._message()
            else:
                await self.uow.commit()
            finally:
                return res

    async def delete_user(self, id: UUID) -> None | str:
        async with self.uow:
            try:
                await self.uow.users.delete_one(filter={"id": id})
            except IntegrityError as e:
                await self.uow.rollback()
                res = e._message()
            else:
                res = None
                await self.uow.commit()
            finally:
                return res

        return None
